% Text Mining with MATLAB 2nd Edition
% www.http://www.textmininglab.net/

% CHAPTER 11: Document Categorization

disp('Example 11.1 -----------------------------------------------------');
clear;                                                             %(11.1a)
load datasets_ch11
whos
books                                                              %(11.1b)
head(dataset,10)                                                   %(11.1c)
pause;

disp('Example 11.2 -----------------------------------------------------');
% resets the random number generator and randomizes the data       %(11.2a)
rng('default');
dataset = dataset(randperm(height(dataset)),:);
% preprocess the data                                              %(11.2b)
docs = tokenizedDocument(dataset.text);
docs = erasePunctuation(docs);
docs = lower(docs);
labels = dataset.book;
nbooks = length(books);
pause;

disp('Example 11.3 -----------------------------------------------------');
hf=figure(1); % creates a new figure                               %(11.3a)
set(hf,'Color',[1 1 1],'Name','Basic description of the Dataset');
% prints subcollection's names and number of documents             %(11.3b)
subplot(2,2,1); axis off 
for k=1:nbooks 
    ndocs = sum(labels==books(k));
    comment = sprintf('%s\nDataset %d (%d documents)\n',books(k),k,ndocs);
    text(0,1.1-k/3,comment);
end
% prints histograms for each subcollection                         %(11.3c)
for k=1:nbooks
    index = labels==books(k);
    chapsizes = doclength(docs(index));
    [freqs,docsizes] = hist(chapsizes,min(chapsizes):max(chapsizes));
    subplot(2,2,k+1);
    bar(docsizes,freqs); 
    axis([61,300,0,25]);
    text(220,23,sprintf('Dataset %d',k));
    xlabel('Document Size (in words)'); ylabel('Frequency');
end
pause;

% GENERATION OF TABLE 11.1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for k=1:nbooks
    fprintf('\nBook title: %s\n',books(k));
    index = labels==books(k);
    fprintf('Paragraphs: %d\n',sum(index));
    fprintf('Running words: %d\n',sum(doclength(docs(index))));
    fprintf('Vocabuary: %d\n',length(docs(index).Vocabulary));
    fprintf('Min doc. size: %d\n',min(doclength(docs(index))));
    fprintf('Max doc. size: %d\n',max(doclength(docs(index))));
    fprintf('Average doc. size: %6.2f\n',mean(doclength(docs(index))));
end
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 11.4 -----------------------------------------------------');
% prepares the test set partition (200 samples)                    %(11.4a)
tstidx = 1:200; % defines the index range
ntst = length(tstidx); % size of the test set
tstdocs = docs(tstidx); % test set documents
tstlbls = labels(tstidx); % test set labels
% prepares the development set partition (200 samples)             %(11.4b)
devidx = 201:400; ndev = length(devidx);
devdocs = docs(devidx); devlbls = labels(devidx);
% prepares the train set partition (1949 samples)                  %(11.4c)
trnidx = 401:length(docs); ntrn = length(trnidx);
trndocs = docs(trnidx); trnlbls = labels(trnidx);
pause;

disp('Example 11.5 -----------------------------------------------------');
trnbow = bagOfWords(trndocs); % computes bag-of-words model        %(11.5a)
% removes infrequent words (counts <= 5)                           %(11.5b) 
trnbow = removeInfrequentWords(trnbow,5);
% removes 40 most frequent words                                   %(11.5c) 
frqwds = topkwords(trnbow,40).Word;
trnbow = removeWords(trnbow,frqwds);
vocab = trnbow.Vocabulary; % stores the vocabulary                 %(11.5d)
vsize = length(vocab); % vocabulary size
pause;

disp('Example 11.6 -----------------------------------------------------');
% computes tfidf matrices for train, development, and test          %(11.6)
trntfidf = tfidf(trnbow);
tsttfidf = tfidf(trnbow,tstdocs);
devtfidf = tfidf(trnbow,devdocs);
pause;

disp('Example 11.7 -----------------------------------------------------');
save datasets books nbooks vocab vsize ntst tstdocs tstlbls tsttfidf ...
ndev devtfidf devdocs devlbls ntrn trndocs trnlbls trnbow trntfidf  %(11.7)
pause;

disp('Example 11.8 -----------------------------------------------------');
% Not a MATLAB script 
pause;
 
disp('Example 11.9 -----------------------------------------------------');
clear; rng('default'); load datasets                                %(11.9)
pause;

disp('Example 11.10 ----------------------------------------------------');
% computes Dunn Index for numbers of clusters between 2 and 10 	  %(11.10a)
for ncls=2:10, n = ncls-1;
    for m=1:5 % computes 5 different simulations for each case
        % applies the k-means clustering algorithm
        [idxs,centroids] = kmeans(trntfidf,ncls,'Distance','cosine');
        % initializes variable for intra-cluster distances 
        indist = zeros(1,ncls);
        for k=1:ncls % computes intra-cluster distances
            temp = 1-cosineSimilarity(trntfidf(idxs==k,:),centroids(k,:));
            indist(k) = max(temp);
        end
        indistmax = max(indist); % maximum intra-cluster distance
        % computes the minimum inter-cluster distance
        outdistmtx = 1-cosineSimilarity(centroids);
        outdistmin = min(outdistmtx(triu(outdistmtx,1)>0));
        % computes the Dunn Index
        dunn(n,m) = outdistmin/indistmax;
        nclusts(n) = ncls;
    end
end
averagedunn = mean(dunn'); % computes Dunn Index averages	      %(11.10b)
pause;

% GENERATION OF FIGURE 3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hf = figure(3);
figtitle = 'Cluster Number Detection with Dunn Index';
set(hf,'Color',[1 1 1],'Name',figtitle);
bar(nclusts,averagedunn);
axis([1,11,0.25,0.45]);
xlabel('Number of Clusters'); ylabel('Dunn Index');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 11.11 ----------------------------------------------------');
% applies k-means (with k=nbooks) to the train set 	              %(11.11a)
[~,centroids] = kmeans(trntfidf,nbooks,'Distance','cosine');
% finds the closest centroid for each sample in the test set      %(11.11b)
[~,idxs] = max(cosineSimilarity(centroids,tsttfidf));
pause;

disp('Example 11.12 ----------------------------------------------------');
hf = figure(4);                                                    %(11.12)
figtitle = 'Cross-plot between cluster indexes and category labels';
set(hf,'Color',[1 1 1],'Name',figtitle);
for n=1:length(tstlbls) % gets book labels of samples in the test set 
    nlbl(n,1) = find(tstlbls(n)==books); 
end
plot(nlbl+randn(size(nlbl))/10,idxs+randn(size(idxs))/10,'.');
xlabel('Actual Category Labels'); ylabel('Cluster Indexes');
xticks([1,2,3]); xticklabels(books);
pause;

disp('Example 11.13 ----------------------------------------------------');
% considers all possible mappings                                 %(11.13a)
permutations = perms([1,2,3]); 
% computes cluster-book overlaps for all possible mappings        %(11.13b)
for n=1:size(permutations,1), temp = 0;
    for k = 1:nbooks
        clusters = idxs'==k;
        permutedbooks = tstlbls==books(permutations(n,k)); 
    	temp = temp + sum(clusters & permutedbooks);
    end
    overlaps(n) = temp; 
end
% gets the best mapping (i.e. maximum overlap)                    %(11.13c)
[~,best] = max(overlaps); 
books(permutations(best,:))
pause;

disp('Example 11.14 ----------------------------------------------------');
% NOT A MATLAB SCRIPT
pause;

disp('Example 11.15 ----------------------------------------------------');
predictions = books(permutations(best,idxs))';                     %(11.15)
pause;

disp('Example 11.16 ----------------------------------------------------');
accuracy = sum(predictions==tstlbls)/ntst*100                      %(11.16)
pause;

disp('Example 11.17 ----------------------------------------------------');
% computes and displays the confusion matrix                       %(11.17)
confusion_mtx = confusionmat(predictions,tstlbls,'ORDER',books);
cmtx = array2table(confusion_mtx,'VariableNames',string(books));
cmtx.('Classified as') = string(books)';
disp(cmtx)
pause;

disp('Example 11.18 ----------------------------------------------------');
accuracy = sum(diag(confusion_mtx))/sum(sum(confusion_mtx))*100    %(11.18)
pause;

disp('Example 11.19 ----------------------------------------------------');
clear; rng('default'); load datasets
pause;

disp('Example 11.20 ----------------------------------------------------');
kvals = 3:10; % considers k values from 3 to 10                   %(11.20a)
for k=1:length(kvals)
    knn_model = fitcknn(trntfidf,trnlbls,...
                        'Distance','cosine','NumNeighbors',kvals(k));
    predictions = predict(knn_model,devtfidf);
    accuracy(k) = sum(devlbls==predictions)/ndev*100;
end
[maxaccuracy,idxoptim] = max(accuracy);                           %(11.20b)
koptim = kvals(idxoptim); % optimum value of k  
fprintf('koptim = %d, maxacc = %5.2f\n',koptim,maxaccuracy)
pause;

% GENERATES FIGURE 11.5 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hf = figure(5);                                                   
figtitle = 'KNN Parameter Search';
set(hf,'Color',[1 1 1],'Name',figtitle);
bar(kvals,accuracy);
axis([kvals(1)-1,kvals(end)+1,90,98]);
xlabel('Number of Nearest Neighbors "K"'); 
ylabel('Accuracy over Development Set');
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 11.21 ----------------------------------------------------');
% trains a new knn model over the train set with k = koptim       %(11.21a)
knn_model = fitcknn(trntfidf,trnlbls,...
                    'Distance','cosine','NumNeighbors',koptim);
% computes accuracy of the generated model over the test set      %(11.21b)
predictions = predict(knn_model,tsttfidf);
accuracy = sum(predictions==tstlbls)/ntst*100
pause;

disp('Example 11.22 ----------------------------------------------------');
confusion_mtx = confusionmat(predictions,tstlbls,'ORDER',books);   %(11.22)
cmtx = array2table(confusion_mtx,'VariableNames',string(books));
cmtx.('Classified as') = string(books)';
disp(cmtx)
pause

disp('Example 11.23 ----------------------------------------------------');
layers = [                                                         %(11.23)
    featureInputLayer(vsize)
    fullyConnectedLayer(10)
    tanhLayer
    fullyConnectedLayer(nbooks)
    softmaxLayer
    classificationLayer];
pause;

disp('Example 11.24 ----------------------------------------------------');
options = trainingOptions('sgdm', ...                              %(11.24)
    'MaxEpochs',500,...
    'MiniBatchSize',64, ...
    'Shuffle','every-epoch', ...
    'ValidationData',{full(devtfidf),devlbls},...
    'ValidationFrequency',10,...
    'ValidationPatience',5,...
    'Verbose',false);
pause;

disp('Example 11.25 ----------------------------------------------------');
% trains with train set and uses dev set for validation           %(11.25a)
mlp_model = trainNetwork(full(trntfidf),trnlbls,layers,options);
% computes accuracy of the generated model over the test set      %(11.25b)
predictions = classify(mlp_model,full(tsttfidf));
acc = sum(tstlbls==predictions)/ntst*100
% computes the confusion matrix                                   %(11.25c)
confusion_mtx = confusionmat(predictions,tstlbls,'ORDER',books);
cmtx = array2table(confusion_mtx,'VariableNames',string(books));
cmtx.('Classified as') = string(books)';
disp(cmtx)
pause;

disp('Example 10.26 ----------------------------------------------------');
for k=1:nbooks                                                     %(11.26)
    % computes the average document vector for the kth category
    kindexes =  trnlbls==books(k);
    meanvect(k,:) = sum(trntfidf(kindexes,:),1)/sum(kindexes);
    % normalizes the computed average vector
    meanvect(k,:) = meanvect(k,:)/norm(meanvect(k,:));
end
pause;

disp('Example 10.27 ----------------------------------------------------');
% gets the 10 most relevant words in category 1 (Oliver Twist)    %(10.27a)
[~,idx1] = sort(meanvect(1,:)-meanvect(2,:)-meanvect(3,:),'descend');
disp(vocab(idx1(1:10)))
% gets the 10 most relevant words in category 2 (Don Quixote)     %(10.27b)
[~,idx2] = sort(meanvect(2,:)-meanvect(1,:)-meanvect(3,:),'descend');
disp(vocab(idx2(1:10)))
% gets the 10 most relevant words in cat 3 (Pride & Prejudice)    %(10.27c)
[~,idx3] = sort(meanvect(3,:)-meanvect(1,:)-meanvect(2,:),'descend');
disp(vocab(idx3(1:10)))
pause;

disp('Example 10.28 ----------------------------------------------------');
% selects the 7 most representative words from each category      %(10.28a)
words = [idx1(1:7),idx2(1:7),idx3(1:7)];
wordmtx = trntfidf(:,words)';
% computes and plots the corresponding dendrogram
hf = figure(7); ha = subplot(1,3,1); 
set(hf,'Color',[1 1 1],'Name','Word dendrograms for different word sets'); 
y = pdist(wordmtx,'cosine'); z = linkage(y,'average'); 
[h,t] = dendrogram(z,0,'labels',vocab(words),'orientation','right');
temp = axis; axis([0,1,temp(3:4)]); set(ha,'Xcolor',[1 1 1]);
ylabel('Words in ranks 1 to 7 for each category');
% selects words in ranks 101 to 107 from each category            %(10.28b)
words = [idx1(101:107),idx2(101:107),idx3(101:107)];
wordmtx = trntfidf(:,words)';
% computes and plots the corresponding dendrogram
ha = subplot(1,3,2); 
y = pdist(wordmtx,'cosine'); z = linkage(y,'average'); 
[h,t] = dendrogram(z,0,'labels',vocab(words),'orientation','right');
temp = axis; axis([0,1,temp(3:4)]); set(ha,'Xcolor',[1 1 1]);
ylabel('Words in ranks 101 to 107 for each category');
% selects words in ranks 1001 to 1007 from each category          %(10.28c)
words = [idx1(1001:1007),idx2(1001:1007),idx3(1001:1007)];
wordmtx = trntfidf(:,words)';
% computes and plots the corresponding dendrogram
ha = subplot(1,3,3); 
y = pdist(wordmtx,'cosine'); z = linkage(y,'average'); 
[h,t] = dendrogram(z,0,'labels',vocab(words),'orientation','right');
temp = axis; axis([0,1,temp(3:4)]); set(ha,'Xcolor',[1 1 1]);
ylabel('Words in ranks 1001 to 1007 for each category');
pause;

disp('Example 11.29 ----------------------------------------------------');
% selects the 7 most representative words from each category       %(11.29)
words = [idx1(1:7),idx2(1:7),idx3(1:7)];
% applies multidimensional scaling to construct a 3-dimensional embedding
map = mdscale(pdist(trntfidf(:,words)','cosine'),3,'criterion','sammon');
% projects the 3-diemnsional word representations on the unit sphere 
for k=1:size(map,1), normap(k,:) = map(k,:)/norm(map(k,:)); end
pause;

disp('Example 11.30 ----------------------------------------------------');
% creates a new figure                                            %(11.30a)
hf = figure(8); set(hf,'Color',[1 1 1],'Name','Word Cloud'); 
% creates and plots a 3-D white semi-transparent sphere           %(11.30b)
[x,y,z] =sphere(20); c = ones(21,21);
surf(x,y,z,c,'EdgeAlpha',0.0,'FaceColor',[1 1 1],'FaceAlpha',0.3); 
axis off; axis square; axis vis3d; 
% makes font sizes proportional to term-frequencies               %(11.30c)
tfmtx = trnbow.Counts;
fsize = sum(tfmtx,1); fsize = 3*ceil(log(full(fsize(words)))');
% assigns a color to each document category                       %(11.30d)
fcolor = char(['r'*ones(7,1);'b'*ones(7,1);'k'*ones(7,1)]);
hold on; % plots the word could                                   %(11.30e)
for k=1:length(words)
    x = 1.5*normap(k,1); y = 1.5*normap(k,2); z = 1.5*normap(k,3);  
    w = vocab(words(k)); fc = fcolor(k); fs = fsize(k);
    text(x,y,z,w,'HorizontalAlignment','center','Color',fc,'Fontsize',fs);
end; hold off;
axis(1.1*[-1,1,-1,1,-1,1]);
pause;

disp('Example 11.31 ----------------------------------------------------');
% captures current view's azimuth and elevation                   %(11.31a)
[ai,ei] = view; 
% generates a trajectory for the azimuth
ai = ai/180*pi; af = ai+2*pi; az = linspace(ai,af,700);
% generates a trajectory for the elevation
ei = ei/180*pi; ef = ei-pi/2; el = linspace(ei,ef,700);
% converts azimuths and elevations into cartesian coordinates  
x = cos(el).*sin(az); y =-cos(el).*cos(az); z = sin(el);
% rotates the view of the word cloud                              %(11.31b)
for k=1:length(x), view([x(k),y(k),z(k)]); pause(0.01); end
pause;

disp('Example 11.32 to 11.35 -------------------------------------------');
% not a MATLAB script
pause;

disp('Example 11.36 ----------------------------------------------------');
clear; rng('default'); load datasets                               %(11.36)
pause;

disp('Example 11.37 ----------------------------------------------------');
ntopics=25; % sets the number of topics                           %(11.37a)
for k = 1:nbooks % computes an LDA model for each book category   %(11.37b)
    counts = encode(trnbow,trndocs(trnlbls==books(k)));
    models(k) = fitlda(counts,ntopics,'Verbose',0);
end
pause;

disp('Example 11.38 ----------------------------------------------------');
logprobs = zeros(length(devdocs),nbooks);    	                   %(11.38)
for n=1:ndev % iterates over the samples in the development set
    counts = encode(trnbow,devdocs(n));
    for k=1:nbooks % iterates over the LDA models
        logprobs(n,k) = logp(models(k),counts);
    end
end
pause;

disp('Example 11.39 ----------------------------------------------------');
for n=1:size(logprobs,1)                                           %(11.39)
    loglirat(n,1) = logprobs(n,1)-(logprobs(n,2)+logprobs(n,3))/2;
    loglirat(n,2) = logprobs(n,2)-(logprobs(n,1)+logprobs(n,3))/2;
    loglirat(n,3) = logprobs(n,3)-(logprobs(n,1)+logprobs(n,2))/2;
end
pause;

disp('Example 11.40 ----------------------------------------------------');
zeta = -100:100; acc = zeros(length(zeta),nbooks);	               %(11.40)                             %(10.40)
for k=1:length(zeta)
    % assigns categories to samples in the development set  
    [~,tmp1] = max((loglirat+repmat([zeta(k) 0 0],ndev,1))');
    [~,tmp2] = max((loglirat+repmat([0 zeta(k) 0],ndev,1))');
    [~,tmp3] = max((loglirat+repmat([0 0 zeta(k)],ndev,1))');
    % computes the classification accuracies over the development set  
    correct = [sum(devlbls==books(tmp1)'),...
               sum(devlbls==books(tmp2)'),sum(devlbls==books(tmp3)')];
    acc(k,:) = correct/ndev*100;
end
pause;

% GENERATES FIGURE 9 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
name = 'Accuracy curves for variying values of the offset parameter';
hf = figure(9); set(hf,'Color',[1 1 1],'Name',name);
plot(zeta,acc(:,1)','r-',zeta,acc(:,2)','b--',zeta,acc(:,3)','k-.')
axis([-100,100,50,100])
legend({'for variying values of \zeta_1',...
        'for variying values of \zeta_2',...
        'for variying values of \zeta_3'},'Location','Southwest')
xlabel('Value of the offset parameter \zeta (log ratio of priors)')
ylabel('Classification accuracy over the development set')
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Example 11.41 ----------------------------------------------------');
[~,temp] = max((loglirat+zeros(ndev,nbooks))');	                   %(11.41)
accuracy = sum(devlbls==books(temp)')/ndev*100
pause;

disp('Example 11.42 ----------------------------------------------------');
% specifies the region of interest	                              %(11.42a)
zeta1 = 0:20; zeta2 = -20:20; zeta3 = -30:0;
ncases = length(zeta1)*length(zeta2)*length(zeta3);
% computes accuracy values within the region of interest	      %(11.42b)
index = 0; acc = zeros(ncases,1); zeta = zeros(ncases,nbooks);  
for k=1:length(zeta1)
    for n=1:length(zeta2)
        for j=1:length(zeta3)
            index = index+1;
            zeta(index,:) = [zeta1(k),zeta2(n),zeta3(j)];
            zeta_mtx = repmat(zeta(index,:),ndev,1);
            [~,temp] = max((loglirat+zeta_mtx)');
            acc(index) = sum(devlbls==books(temp)')/ndev*100;
        end
    end   
end
% finds the case with maximum accurracy                           %(11.42c)
[maxacc,index] = max(acc)
% gets the optimum values of zeta1, zeta2 and zeta3               %(11.42d)
optzeta = zeta(index,:)
pause;

disp('Example 11.43 ----------------------------------------------------');
sum(acc==maxacc)                                                   %(11.43)
pause; 

disp('Example 11.44 ----------------------------------------------------');
logprobs = zeros(ntst,nbooks);    	                              %(11.44a)
for n=1:ntst % iterates over the samples in the test set
    counts = encode(trnbow,tstdocs(n));
    for k=1:nbooks % iterates over the LDA models
        logprobs(n,k) = logp(models(k),counts);
    end
end 
for n=1:size(logprobs,1)                                          %(11.44b)
    loglirat(n,1) = logprobs(n,1)-(logprobs(n,2)+logprobs(n,3))/2;
    loglirat(n,2) = logprobs(n,2)-(logprobs(n,1)+logprobs(n,3))/2;
    loglirat(n,3) = logprobs(n,3)-(logprobs(n,1)+logprobs(n,2))/2;
end
pause;

disp('Example 11.45 ----------------------------------------------------');
zeta_mtx = repmat(optzeta,ntst,1);                                %(11.45a)
[~,idxs] = max((loglirat+zeta_mtx)');
predictions = books(idxs)';                                       
accuracy = sum(predictions==tstlbls)/ntst*100
confusion_mtx = confusionmat(predictions,tstlbls,'ORDER',books);  %(11.45b)
cmtx = array2table(confusion_mtx,'VariableNames',string(books));
cmtx.('Classified as') = string(books)';
disp(cmtx)
pause;


disp('Example 11.46 ----------------------------------------------------');
rng('default'); % restarts the random number generator             %(11.46)
ldamodel = fitlda(trnbow,ntopics,'Verbose',0); % computes the LDA model
pause;

disp('Example 11.47 ----------------------------------------------------');
trnfeatmtx = transform(ldamodel,trndocs);                          %(11.47)
tstfeatmtx = transform(ldamodel,tstdocs);
devfeatmtx = transform(ldamodel,devdocs);
pause;

disp('Example 11.48 ----------------------------------------------------');
whos trnfeatmtx tstfeatmtx devfeatmtx                             %(11.48a)
sum(trnfeatmtx(1,:)) % sum of topic scores for train document 1   %(11.48b)
pause;

disp('Example 11.49 ----------------------------------------------------');
% defines the MLP architecture                                    %(11.49a)
layers = [
    featureInputLayer(ntopics)
    fullyConnectedLayer(10)
    tanhLayer
    fullyConnectedLayer(nbooks)
    softmaxLayer
    classificationLayer];
% sets the training options                                       %(11.49b)
options = trainingOptions('sgdm', ...
    'MaxEpochs',500,...
    'MiniBatchSize',64, ...
    'Shuffle','every-epoch', ...
    'ValidationData',{devfeatmtx,devlbls},...
    'ValidationFrequency',10,...
    'ValidationPatience',5,...
    'Verbose',false);
% trains the MLP classifier                                       %(11.49c)
mlpnet = trainNetwork(trnfeatmtx,trnlbls,layers,options);
% computes predictions and accuracy over the test set             %(11.49d)
predictions = classify(mlpnet,tstfeatmtx);
acc = sum(predictions==tstlbls)/ntst*100 
% computes and displays the confusion matrix                      %(11.49e)
confusion_mtx = confusionmat(predictions,tstlbls,'ORDER',books);
cmtx = array2table(confusion_mtx,'VariableNames',string(books));
cmtx.('Classified as') = string(books)';
disp(cmtx)
pause;

disp('Example 11.50 ----------------------------------------------------');
name = 'Distribution of Topics over Document Categories';          %(11.50)
hf = figure(10);
set(hf,'Color',[1 1 1],'Name',name);
for k=1:nbooks
    x = predict(ldamodel,trndocs(trnlbls==books(k)));
    subplot(nbooks,1,k)
    histogram(x,'BinMethod','integers','Normalization','probability')
    xlabel(books(k));
    axis([0,ntopics+1,0,0.4]);
end
pause;

disp('Example 11.51 ----------------------------------------------------');
topics = [24,17,8,1,4];                                            %(11.51)
tbl = table();
for k=1:length(topics)
    colname = sprintf('Topic %d',topics(k));
    tbl.(colname) = topkwords(ldamodel,10,topics(k)).Word;
end
disp(tbl)
pause;
